@@ -33,6 +33,7 @@
 #include "common/attributes.h"
 
 #include "dav1d/common.h"
+#include "dav1d/dav1d.h"
 
 #if ARCH_AARCH64 || ARCH_ARM
 #include "src/arm/cpu.h"
@@ -42,7 +43,9 @@
 #include "src/x86/cpu.h"
 #endif
 
+void dav1d_init_cpu(void);
 unsigned dav1d_get_cpu_flags(void);
-DAV1D_API void dav1d_set_cpu_flags_mask(const unsigned mask);
+DAV1D_API void dav1d_set_cpu_flags_mask(unsigned mask);
+int dav1d_num_logical_processors(Dav1dContext *c);
 
 #endif /* DAV1D_SRC_CPU_H */
