@@ -37,12 +37,12 @@
 #define DEBUG_B_PIXELS 0
 
 #define decl_recon_b_intra_fn(name) \
-void (name)(Dav1dTileContext *t, enum BlockSize bs, \
+void (name)(Dav1dTaskContext *t, enum BlockSize bs, \
             enum EdgeFlags intra_edge_flags, const Av1Block *b)
 typedef decl_recon_b_intra_fn(*recon_b_intra_fn);
 
 #define decl_recon_b_inter_fn(name) \
-int (name)(Dav1dTileContext *t, enum BlockSize bs, const Av1Block *b)
+int (name)(Dav1dTaskContext *t, enum BlockSize bs, const Av1Block *b)
 typedef decl_recon_b_inter_fn(*recon_b_inter_fn);
 
 #define decl_filter_sbrow_fn(name) \
@@ -50,11 +50,11 @@ void (name)(Dav1dFrameContext *f, int sby)
 typedef decl_filter_sbrow_fn(*filter_sbrow_fn);
 
 #define decl_backup_ipred_edge_fn(name) \
-void (name)(Dav1dTileContext *t)
+void (name)(Dav1dTaskContext *t)
 typedef decl_backup_ipred_edge_fn(*backup_ipred_edge_fn);
 
 #define decl_read_coef_blocks_fn(name) \
-void (name)(Dav1dTileContext *t, enum BlockSize bs, const Av1Block *b)
+void (name)(Dav1dTaskContext *t, enum BlockSize bs, const Av1Block *b)
 typedef decl_read_coef_blocks_fn(*read_coef_blocks_fn);
 
 decl_recon_b_intra_fn(dav1d_recon_b_intra_8bpc);
@@ -65,10 +65,12 @@ decl_recon_b_inter_fn(dav1d_recon_b_inter_16bpc);
 
 decl_filter_sbrow_fn(dav1d_filter_sbrow_8bpc);
 decl_filter_sbrow_fn(dav1d_filter_sbrow_16bpc);
-decl_filter_sbrow_fn(dav1d_filter_sbrow_deblock_8bpc);
-decl_filter_sbrow_fn(dav1d_filter_sbrow_deblock_16bpc);
-decl_filter_sbrow_fn(dav1d_filter_sbrow_cdef_8bpc);
-decl_filter_sbrow_fn(dav1d_filter_sbrow_cdef_16bpc);
+decl_filter_sbrow_fn(dav1d_filter_sbrow_deblock_cols_8bpc);
+decl_filter_sbrow_fn(dav1d_filter_sbrow_deblock_cols_16bpc);
+decl_filter_sbrow_fn(dav1d_filter_sbrow_deblock_rows_8bpc);
+decl_filter_sbrow_fn(dav1d_filter_sbrow_deblock_rows_16bpc);
+void dav1d_filter_sbrow_cdef_8bpc(Dav1dTaskContext *tc, int sby);
+void dav1d_filter_sbrow_cdef_16bpc(Dav1dTaskContext *tc, int sby);
 decl_filter_sbrow_fn(dav1d_filter_sbrow_resize_8bpc);
 decl_filter_sbrow_fn(dav1d_filter_sbrow_resize_16bpc);
 decl_filter_sbrow_fn(dav1d_filter_sbrow_lr_8bpc);
