@@ -110,7 +110,7 @@ extern const int8_t dav1d_cdef_directions[12][2];
 extern const uint16_t dav1d_sgr_params[16][2];
 extern const uint8_t dav1d_sgr_x_by_x[256];
 
-extern const int8_t dav1d_mc_subpel_filters[5+ARCH_X86_64][15][8];
+extern const int8_t dav1d_mc_subpel_filters[6][15][8];
 extern const int8_t dav1d_mc_warp_filter[193][8];
 extern const int8_t dav1d_resize_filter[64][8];
 
